# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re


class Version(object):
    MAJOR, MINOR, PATCH = range(3)

    PATTERN = re.compile(r'(?P<major>\d+)\.(?P<minor>\d+)\.(?P<patch>\d+)(\.(?P<suffix>[\w\d.]+))?')
    FORMAT = r'{major}.{minor}.{patch}'
    FORMAT_SUFFIXED = r'{major}.{minor}.{patch}.{suffix}'

    def __init__(self, major=0, minor=0, patch=0, suffix=None):
        self.major = int(major)
        self.minor = int(minor)
        self.patch = int(patch)
        self.suffix = suffix

    def bump(self, part=None, unsuffix=True, suffix=None):
        if part is Version.MAJOR:
            self.major += 1
            self.minor = 0
            self.patch = 0
        elif part is Version.MINOR:
            self.minor += 1
            self.patch = 0
        elif part is Version.PATCH:
            self.patch += 1

        if unsuffix:
            self.suffix = None
        if suffix:
            self.suffix = suffix

    def copy(self, **kwargs):
        version = Version(**self.__dict__)
        unsuffix = kwargs.pop('unsuffix', False)
        version.bump(unsuffix=unsuffix, **kwargs)
        return version

    def __unicode__(self):
        pattern = self.FORMAT_SUFFIXED if self.suffix else self.FORMAT
        return pattern.format(**self.__dict__)

    @classmethod
    def parse(cls, string):
        match = cls.PATTERN.match(string)
        return cls(**match.groupdict())

    __str__ = __unicode__

    def __repr__(self):
        return "'Version({major},{minor},{patch},{suffix})'".format(**self.__dict__)

    def __eq__(self, other):
        if not isinstance(other, Version):
            return False
        return self.__dict__ == other.__dict__


PARTS = {
    'major': Version.MAJOR,
    'minor': Version.MINOR,
    'patch': Version.PATCH,
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

import os
import sys
import logging

from logging import Formatter, StreamHandler, DEBUG, INFO

__all__ = (
    'init',
)

RESET_TERM = '\033[0;m'

COLOR_CODES = {
    'red': 31,
    'green': 32,
    'yellow': 33,
    'blue': 34,
    'magenta': 35,
    'cyan': 36,
    'white': 37,
    'bgred': 41,
    'bggrey': 100,
}

LEVEL_COLORS = {
    'DEBUG': 'blue',
    'WARNING': 'yellow',
    'ERROR': 'red',
    'CRITICAL': 'bgred',
}

DRYRUN = 25
DIFF = 15


def ansi(color, text):
    """Wrap text in an ansi escape sequence"""
    code = COLOR_CODES[color]
    return '\033[1;{0}m{1}{2}'.format(code, text, RESET_TERM)


class ANSIFormatter(Formatter):
    """
    Convert a `logging.LogRecord' object into colored text, using ANSI
    escape sequences.
    """
    def format(self, record):
        msg = record.getMessage()
        if record.levelname == 'INFO':
            return ansi('cyan', '-> ') + msg
        elif record.levelname == 'DRYRUN':
            return ansi('magenta', 'dryrun-> ') + msg
        elif record.levelname == 'DIFF':
            if msg.startswith('+'):
                return ansi('green', msg)
            elif msg.startswith('-'):
                return ansi('red', msg)
            else:
                return msg
        else:
            color = LEVEL_COLORS.get(record.levelname, 'white')
            return ansi(color, record.levelname.lower()) + ': ' + msg


class TextFormatter(Formatter):
    """
    Convert a `logging.LogRecord' object into text.
    """
    def format(self, record):
        if not record.levelname or record.levelname in ('INFO', 'DIFF'):
            return record.getMessage()
        elif record.levelname == 'DRYRUN':
            return 'dryrun-> {0}'.format(record.getMessage())
        else:
            return record.levelname.lower() + ': ' + record.getMessage()


class BumprLogger(logging.Logger):
    def dryrun(self, *args):
        self.log(DRYRUN, *args)

    def diff(self, *args):
        self.log(DIFF, *args)


def declare():
    logging.addLevelName(DRYRUN, 'DRYRUN')
    logging.addLevelName(DIFF, 'DIFF')
    logging.setLoggerClass(BumprLogger)


def init(level=INFO):
    declare()

    logger = logging.getLogger()
    handler = StreamHandler()

    if (os.isatty(sys.stdout.fileno()) and not sys.platform.startswith('win')):
        fmt = ANSIFormatter()
    else:
        fmt = TextFormatter()
    handler.setFormatter(fmt)
    logger.addHandler(handler)

    if level:
        logger.setLevel(level)


if __name__ == '__main__':  # pragma: no cover
    init(level=DEBUG)

    logger = logging.getLogger(__name__)
    logger.debug('debug')
    logger.dryrun('dryrun')
    logger.diff('diff')
    logger.diff('+ diff')
    logger.diff('- diff')
    logger.info('info')
    logger.warning('warning')
    logger.error('error')
    logger.critical('critical')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys

from setuptools import setup, find_packages


def rst(filename):
    '''
    Load rst file and sanitize it for PyPI.
    Remove unsupported github tags:
     - code-block directive
     - travis ci build badge
    '''
    return open(filename).read()


long_description = '\n'.join((
    rst('README.rst'),
    rst('CHANGELOG.rst'),
    ''
))


install_requires = []
tests_require = ['mock', 'pytest', 'pytest-catchlog']
doc_require = ['sphinx']
qa_require = ['pytest-cov', 'coverage']

if sys.version_info[0:2] < (2, 7):
    install_requires.append('argparse')
    qa_require.append('flake8<3.0')
else:
    qa_require.append('flake8')


setup(
    name='bumpr',
    version=__import__('bumpr.__about__').__version__,
    description=__import__('bumpr.__about__').__description__,
    long_description=long_description,
    url='https://github.com/noirbizarre/bumpr',
    author='Axel Haustant',
    author_email='noirbizarre+github@gmail.com',
    packages=find_packages(),
    install_requires=install_requires,
    license='LGPL',
    use_2to3=True,
    entry_points={
        'console_scripts': [
            'bumpr = bumpr:main',
        ],
    },
    tests_require=tests_require,
    extras_require={
        'test': tests_require,
        'doc': doc_require,
        'qa': qa_require,
    },
    keywords='version bump release tag',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: System :: Software Distribution",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Libraries :: Python Modules",
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import argparse
import logging
import sys

from os.path import exists

try:
    from ConfigParser import RawConfigParser
except ImportError:
    from configparser import RawConfigParser

from bumpr.helpers import ObjectDict
from bumpr.hooks import HOOKS
from bumpr.version import Version, PARTS

logger = logging.getLogger(__name__)
IS_PY3 = sys.version_info[0] == 3

DEFAULTS = {
    'file': None,
    'regex': r'(__version__|VERSION)\s*=\s*(\'|")(?P<version>.+?)(\'|")',
    'encoding': 'utf8',
    'vcs': None,
    'commit': True,
    'tag': True,
    'tag_pattern': '{version}',
    'push': False,
    'verbose': False,
    'dryrun': False,
    'clean': None,
    'tests': None,
    'skip_tests': False,
    'publish': None,
    'bump_only': False,
    'prepare_only': False,
    'files': [],

    'bump': {
        'unsuffix': True,
        'suffix': None,
        'part': None,
        'message': 'Bump version {version}',
    },

    'prepare': {
        'unsuffix': False,
        'suffix': None,
        'part': None,
        'message': 'Update to version {version} for next development cycle',
    },
}


class ValidationError(ValueError):
    pass


class BumprConfigParser(RawConfigParser):
    '''
    A config parser with optionnal implicit `bumpr:` prefix on sections.

    Allow better isolation in setup.cfg.
    '''
    prefix = 'bumpr'

    def candidate_sections(self, section):
        return [section, '{0}:{1}'.format(self.prefix, section)]

    def has_section(self, section):
        sections = self.candidate_sections(section)
        return any(RawConfigParser.has_section(self, section) for section in sections)

    def options(self, section):
        for section in self.candidate_sections(section):
            if RawConfigParser.has_section(self, section):
                return RawConfigParser.options(self, section)

    def has_option(self, section, option):
        sections = self.candidate_sections(section)
        return any(RawConfigParser.has_option(self, section, option) for section in sections)

    def get(self, section, option, **kwargs):
        for section in self.candidate_sections(section):
            if RawConfigParser.has_option(self, section, option):
                return RawConfigParser.get(self, section, option)

    def getboolean(self, section, option):
        for section in self.candidate_sections(section):
            if RawConfigParser.has_option(self, section, option):
                return RawConfigParser.getboolean(self, section, option)

    def items(self, section):
        for section in self.candidate_sections(section):
            if RawConfigParser.has_section(self, section):
                return RawConfigParser.items(self, section)


class Config(ObjectDict):
    def __init__(self, source=None, parsed_args=None):
        super(Config, self).__init__(DEFAULTS)
        for hook in HOOKS:
            self[hook.key] = False

        if source:
            for hook in HOOKS:
                if hook.key in source:
                    self.merge({hook.key: hook.defaults})
            self.merge(source)

        if exists('setup.cfg'):
            self.override_from_config('setup.cfg')

        if parsed_args:
            if 'config' in parsed_args and exists(parsed_args.config):
                self.override_from_config(parsed_args.config)
            self.override_from_args(parsed_args)

    def override_from_config(self, filename):
        config = BumprConfigParser()
        if IS_PY3:
            config.read_file(open(filename))
        else:
            config.readfp(open(filename))

        # Common options
        if config.has_section('bumpr'):
            for option in config.options('bumpr'):
                if option in ('tag', 'commit', 'push', 'bump_only', 'prepare_only', 'skip_tests'):
                    self[option] = config.getboolean('bumpr', option)
                elif option == 'files':
                    self.files = [name.strip() for name in config.get('bumpr', 'files').split('\n') if name.strip()]
                else:
                    self[option] = config.get('bumpr', option)

        # Bump and next section
        for section in 'bump', 'prepare':
            for option in 'message', 'suffix':
                if config.has_option(section, option):
                    self[section][option] = config.get(section, option)
            if config.has_option(section, 'unsuffix'):
                self[section]['unsuffix'] = config.getboolean(section, 'unsuffix')

            if config.has_option(section, 'part'):
                self[section]['part'] = PARTS[config.get(section, 'part').lower()]

        for hook in HOOKS:
            if config.has_section(hook.key):
                if not self.get(hook.key, False):
                    self[hook.key] = hook.defaults
                self[hook.key].update(config.items(hook.key))
            else:
                self[hook.key] = False

    def override_from_args(self, parsed_args):
        for arg in 'file', 'vcs', 'files':
            if arg in parsed_args and getattr(parsed_args, arg) not in (None, [], tuple()):
                self[arg] = getattr(parsed_args, arg)

        for arg in 'verbose', 'dryrun':
            if arg in parsed_args and getattr(parsed_args, arg):
                self[arg] = True

        if hasattr(parsed_args, 'nocommit'):
            self.commit = not parsed_args.nocommit
        for attr in 'bump_only' 'prepare_only', 'push', 'skip_tests':
            if hasattr(parsed_args, attr):
                self[attr] = getattr(parsed_args, attr)

        # Bump
        if parsed_args.part is not None:
            self.bump.part = parsed_args.part
        if parsed_args.suffix is not None:
            self.bump.suffix = parsed_args.suffix
        if parsed_args.unsuffix is not None:
            self.bump.unsuffix = parsed_args.unsuffix

        # Next
        if parsed_args.prepare_part is not None:
            self.prepare.part = parsed_args.prepare_part
        if parsed_args.prepare_suffix is not None:
            self.prepare.suffix = parsed_args.prepare_suffix
        if parsed_args.prepare_unsuffix is not None:
            self.prepare.unsuffix = parsed_args.prepare_unsuffix

    def validate(self):
        if not self.file:
            raise ValidationError('A file is required from the configuration file or the command line')

    @classmethod
    def parse_args(cls, args=None):
        from bumpr import __version__, __description__
        parser = argparse.ArgumentParser(description=__description__)

        parser.add_argument('file', help='Versionned module file', nargs='?')
        parser.add_argument('files', help='Files to update', nargs='*')

        parser.add_argument('--version', action='version', version=__version__)
        parser.add_argument('-v', '--verbose', dest='verbose', action='store_true', help="Verbose output")
        parser.add_argument('-c', '--config', default='bumpr.rc', help='Specify a configuration file')
        parser.add_argument('-d', '--dryrun', action='store_true', help='Do not write anything and display a diff')
        parser.add_argument('-st', '--skip-tests', action='store_true', default=argparse.SUPPRESS, help='Skip tests')

        group = parser.add_mutually_exclusive_group()
        group.add_argument('-b', '--bump', dest='bump_only', action='store_true', default=argparse.SUPPRESS,
                           help='Only perform the bump')
        group.add_argument('-pr', '--prepare', dest='prepare_only', action='store_true', default=argparse.SUPPRESS,
                           help='Only perform the prepare')

        # Bump behavior for bump
        group = parser.add_argument_group('bump')
        group.add_argument('-M', '--major', dest='part', action='store_const', const=Version.MAJOR,
                           help="Bump major version")
        group.add_argument('-m', '--minor', dest='part', action='store_const', const=Version.MINOR,
                           help="Bump minor version")
        group.add_argument('-p', '--patch', dest='part', action='store_const',
                           const=Version.PATCH, help="Bump patch version")
        group.add_argument('-s', '--suffix', dest='suffix', type=str, help="Set suffix")
        group.add_argument('-u', '--unsuffix', dest='unsuffix', action='store_true', default=None, help="Unset suffix")

        # Bump behavior for prepare version
        group = parser.add_argument_group('prepare')
        group.add_argument('-pM', '--prepare-major', dest='prepare_part', action='store_const',
                           const=Version.MAJOR, help="Bump major version")
        group.add_argument('-pm', '--prepare-minor', dest='prepare_part', action='store_const',
                           const=Version.MINOR, help="Bump minor version")
        group.add_argument('-pp', '--prepare-patch', dest='prepare_part', action='store_const',
                           const=Version.PATCH, help="Bump patch version")
        group.add_argument('-ps', '--prepare-suffix', dest='prepare_suffix', type=str, help="Set suffix")
        group.add_argument('-pu', '--prepare-unsuffix', dest='prepare_unsuffix', action='store_true',
                           help="Unset suffix")

        group = parser.add_argument_group('Version control system')
        group.add_argument('--vcs', choices=['git', 'hg'], default=None, help='VCS implementation')
        group.add_argument('-nc', '--nocommit', action='store_true', default=argparse.SUPPRESS,
                           help='Do not commit')
        group.add_argument('-P', '--push', action='store_true', default=argparse.SUPPRESS,
                           help='Push changes to remote repository')
        group.add_argument('-nP', '--no-push', action='store_false', dest='push', default=argparse.SUPPRESS,
                           help='Don\'t push changes to remote repository')

        parsed_args = parser.parse_args(args)
        return cls(parsed_args=parsed_args)



# -*- coding: utf-8 -*-
from os.path import isdir

from bumpr.helpers import execute, BumprError


class BaseVCS(object):
    def __init__(self, verbose=False):
        self.verbose = verbose

    def execute(self, command):
        '''Execute a command'''
        execute(command, verbose=self.verbose)

    def validate(self):
        '''Ensure the working dir is a repository and there is no modified files'''
        raise NotImplementedError

    def commit(self, message):
        '''Commit all modified files'''
        raise NotImplementedError

    def tag(self, name):
        '''Create a tag'''
        raise NotImplementedError

    def push(self):
        '''Push changes to remote repository'''
        raise NotImplementedError


class Git(BaseVCS):
    def validate(self):
        if not isdir('.git'):
            raise BumprError('Current directory is not a git repopsitory')

        for line in execute('git status --porcelain', verbose=False).splitlines():
            if not line.startswith('??'):
                raise BumprError('The current repository contains modified files')

    def commit(self, message):
        self.execute(["git", "commit", "-am", message])

    def tag(self, name):
        self.execute(["git", "tag", name])

    def push(self):
        self.execute(["git", "push"])
        self.execute(["git", "push", "--tags"])


class Mercurial(BaseVCS):
    def validate(self):
        if not isdir('.hg'):
            raise BumprError('Current directory is not a mercurial repopsitory')

        for line in execute('hg status -mard', verbose=False).splitlines():
            if not line.startswith('??'):
                raise BumprError('The current repository contains modified files')

    def commit(self, message):
        self.execute(["hg", "commit", "-A", "-m", message])

    def tag(self, name):
        self.execute(["hg", "tag", name])

    def push(self):
        self.execute(["hg", "push"])


class Bazaar(BaseVCS):
    def validate(self):
        if not isdir('.bzr'):
            raise BumprError('Current directory is not a bazaar repopsitory')

        for line in execute('bzr status --short', verbose=False).splitlines():
            if not line.startswith('?'):
                raise BumprError('The current repository contains modified files')

    def commit(self, message):
        self.execute(["bzr", "commit", "-m", message])

    def tag(self, name):
        self.execute(["bzr", "tag", name])

    def push(self):
        self.execute(["bzr", "push"])


class Fake(BaseVCS):
    def validate(self):
        return True


VCS = {
    'git': Git,
    'hg': Mercurial,
    'bzr': Bazaar,
    'fake': Fake,
}
